#ifndef _BOSS_H
#define _BOSS_H
/* ----------------------------------------------------------------------
 * FILE: boss.h
 * PACKAGE: boss - 
 * DESCRIPTION:
 *	Public header file for BossXXXXX() library routines.
 *
 * REVISION HISTORY:
 *
 * AUTHOR:
 *	Ken Stauffer. December 1995
 */

/*----------------------------------------------------------------------
 * 1. To store a normal date:
 *	Set year, month, day to desired values.
 *
 * 2. To store a reminder date:
 *				year	month	day
 *	Yearly reminder:	0	mm	dd
 *	Monthly reminder:	0	0	dd
 *	Daily reminder:		0	0	0
 *
 * 3. To encode "nodate":
 *	year = 0, month = 0, day = 0
 *
 */
typedef struct {
	int	year;			/* eg 1995, 2010, ... */
	int	month;			/* 1-12 */
	int	day;			/* 1-31 */
} BOSS_DATE;

/*----------------------------------------------------------------------
 * 1. To store a normal time spec:
 *	Set hour, minute, pm to desired values.
 *
 * 2. To store "notime":
 *	Set hour, minute and pm to -1.
 */
typedef struct {
	int	hour;
	int	minute;
	int	pm;		/* 0 - am, 1 - pm */
} BOSS_TIME;

typedef struct boss_phone {
	char			*name;
	char			*number;
	char			*address;
	char			*free1;
	char			*free2;
	char			*free3;
	char			*free4;
	char			*free5;
	char			*free6;
	struct boss_phone	*next;
} BOSS_PHONE;

typedef struct boss_memo {
	char			*title;
	char			*body;
	struct boss_memo	*next;
} BOSS_MEMO;

typedef struct boss_schedule {
	BOSS_DATE		date;
	BOSS_TIME		start;
	BOSS_TIME		end;
	BOSS_TIME		alarm;
	char			*description;
	struct boss_schedule	*next;
} BOSS_SCHEDULE;

typedef struct boss_reminder {
	BOSS_DATE		date;
	BOSS_TIME		alarm;
	char			*description;
	struct boss_reminder	*next;
} BOSS_REMINDER;

typedef struct boss_calendar {
	BOSS_DATE		date;
	unsigned long		days;
	struct boss_calendar	*next;
} BOSS_CALENDAR;

typedef struct boss_todo {
	int			checked;
	BOSS_DATE		date_stamp;
	BOSS_TIME		time_stamp;
	char			*description;
	struct boss_todo	*next;
} BOSS_TODO;

typedef struct boss_expense {
	double			amount;
	BOSS_DATE		date;
	char			*payment;
	char			*expense;
	char			*description;
	struct boss_expense	*next;
} BOSS_EXPENSE;

typedef struct {
	BOSS_PHONE	*phone;
	BOSS_MEMO	*memo;
	BOSS_SCHEDULE	*schedule;
	BOSS_REMINDER	*reminder;
	BOSS_CALENDAR	*calendar;
	BOSS_TODO	*todo;
	BOSS_EXPENSE	*expense;
} BOSS_DATA;

/*
 * Prototypes
 */
extern char		*BossError(void);
extern BOSS_DATA	*BossParse(char *);
extern int		BossGenerate(char *, BOSS_DATA *);

extern char		*BossDecodeStr(unsigned char *);
extern unsigned char	*BossEncodeStr(char *);

extern void		BossSetIOCB( void (*)(char *, int) );
extern int		BossTransmit(char *, int, char *, BOSS_DATA *);
extern BOSS_DATA	*BossReceive(char *, int, char *);
extern void		BossStop(void);

extern void		BossFree(BOSS_DATA *);
extern BOSS_DATA	*BossUnion(BOSS_DATA *, BOSS_DATA *);
extern BOSS_DATA	*BossDiff(BOSS_DATA *, BOSS_DATA *);

#endif