/* ----------------------------------------------------------------------
 * FILE: lex.h
 * PACKAGE: boss - 
 * DESCRIPTION:
 *	Private header file for BossXXXX() library routines.
 *
 * REVISION HISTORY:
 *
 * AUTHOR:
 *	Ken Stauffer. December 1995
 */

typedef struct {
	BOSS_DATE	date;
	BOSS_TIME	time;
} DATETIME;

typedef struct {
	BOSS_TIME	start;
	BOSS_TIME	end;
} RANGE;

typedef struct strlst {
	char		*str;
	struct strlst	*next;
} STRLST;

typedef struct numlst {
	int		num;		/* (-1) indicates highlight */
	struct numlst	*next;
} NUMLST;

/* ----------------------------------------------------------------------
 * yacc stack stuff:
 */
union ystack {
	unsigned long	value;
	double		fvalue;
	char		*str;
	BOSS_DATE	date;
	BOSS_TIME	time;
	DATETIME	dt;
	STRLST		*strlst;
	NUMLST		*numlst;
	RANGE		range;

	BOSS_DATA	*data;
	BOSS_PHONE	*phone;
	BOSS_MEMO	*memo;
	BOSS_SCHEDULE	*schedule;
	BOSS_REMINDER	*reminder;
	BOSS_CALENDAR	*calendar;
	BOSS_TODO	*todo;
	BOSS_EXPENSE	*expense;
};

#define ISWHITE(c)		((c)=='\t' || (c)==' ' || (c)=='\n')
#define MAX(a,b)		( ((a)>(b))?(a):(b) )
#define KEYWORDCMP(a,b)		strcasecmp((a),(b))
#define DEFTABSIZE		8
#define CB(s,n)			{ if(Boss.cb) (Boss.cb)((s),(n)); }

/*
 * Macros for converting from 24-hour time to 12-hour time.
 */
#define HOUR24TOAMPM(h)		((h)>12)
#define HOUR24TO12(h)		(((h)>12) ? ((h)-12) : (((h)==0) ? 12 : (h)))
#define HOUR12TO24(h,pm)	((pm) ? (((h)+12)%24) : (h))

enum { T_AM, T_PM, T_NOTSPECIFIED };

/*
 * Globals for the Boss-library.
 */
struct boss_globals {
	int		lineno;
	int		colpos;
	int		tabsize;
	int		lastpos;
	jmp_buf		main_env;
	FILE		*fp;
	char		*datafile;
	char		error[ 1024 ];
	void		(*cb)(char *, int);
	int		stop;
	int		transmit_mode;
	int		counter;
	int		offset;
	void		(*intr)(void);

	BOSS_DATA	*data;
};

typedef struct boss_record {
	unsigned int		nbytes;
	unsigned int		flag;
	unsigned int		type;
	unsigned int		offset;
	unsigned int		checksum;
	unsigned int		calculated;
	unsigned char		buf[256];

	struct boss_record	*next;
} BOSS_RECORD;

#define BOSS_SECTION_COUNT	8	/* how many section types exist */
typedef enum {
	S_UNKNOWN,	S_PHONE,	S_MEMO,	S_REMINDER,
	S_SCHEDULE,	S_CALENDAR,	S_TODO,	S_EXPENSE
} BOSS_SECTION;

/*
 * Prototypes
 */
extern struct boss_charset	BossCharset[];
extern struct boss_globals	Boss;
extern union ystack		bosslval;
extern void			bosserror(char *s);
extern void			boss_error(char *, ...);
extern char			*boss_strerror(int);

extern void phone_add(BOSS_PHONE *);
extern void memo_add(BOSS_MEMO *);
extern void schedule_add(BOSS_SCHEDULE *);
extern void reminder_add(BOSS_REMINDER *);
extern void calendar_add(BOSS_CALENDAR *);
extern void todo_add(BOSS_TODO *);
extern void expense_add(BOSS_EXPENSE *);
